package com.myco.myapp.fix;

import static quickfix.SessionFactoryTestSupport.createResetTestSession;

import java.io.IOException;
import java.util.Date;

import junit.framework.Assert;

import org.junit.Test;

import quickfix.DefaultMessageFactory;
import quickfix.FixVersions;
import quickfix.Message;
import quickfix.Responder;
import quickfix.SessionID;
import quickfix.SystemTime;
import quickfix.UnitTestApplication;
import quickfix.SessionFactoryTestSupport.ResetTestSession;
import quickfix.field.BeginString;
import quickfix.field.EncryptMethod;
import quickfix.field.HeartBtInt;
import quickfix.field.MsgSeqNum;
import quickfix.field.MsgType;
import quickfix.field.SenderCompID;
import quickfix.field.SendingTime;
import quickfix.field.TargetCompID;
import quickfix.field.TestReqID;
import quickfix.fix44.TestRequest;

public class SessionResetTest {

	 
	 @Test
	    public void testSessionResetDeadlock() throws Exception {

	        final UnitTestApplication application = new UnitTestApplication();

	        final SessionID sessionID = new SessionID(FixVersions.BEGINSTRING_FIX44, "SENDER", "TARGET");
	        final ResetTestSession session = createResetTestSession(sessionID, application, true, true);

	        
	        final UnitTestResponder responder = new UnitTestResponder();
	        session.setResponder(responder);

	        session.logon();
	        session.next();
	        
	        final Message logonRequest = new Message(responder.sentMessageData);
	        final Message logonResponse = new DefaultMessageFactory().create(
	                sessionID.getBeginString(), MsgType.LOGON);
	        logonResponse.setInt(EncryptMethod.FIELD, EncryptMethod.NONE_OTHER);
	        logonResponse.setInt(HeartBtInt.FIELD, logonRequest.getInt(HeartBtInt.FIELD));
	        final Message.Header header = logonResponse.getHeader();

	        header.setString(BeginString.FIELD, sessionID.getBeginString());
	        header.setString(SenderCompID.FIELD, sessionID.getSenderCompID());
	        header.setString(TargetCompID.FIELD, sessionID.getTargetCompID());
	        header.setInt(MsgSeqNum.FIELD, 1);
	        header.setUtcTimeStamp(SendingTime.FIELD, SystemTime.getDate(), true);
	        
	        session.handbrakeOn();
	        Thread t1 = new Thread(new Runnable() {				
				@Override
				public void run() {
					try {
						session.reset();
					} catch (IOException e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}				
				}
			},"SessionReset");
	        
	        
	        
	        Thread t = new Thread(new Runnable() {				
				@Override
				public void run() {
					session.send(createAdminMessage(2));
					session.send(createAdminMessage(3));	
				}
			},"SessionSend");
	        t1.start();
	        //Thread.currentThread().wait(1000);
	        t.start();
	        //Thread.currentThread().wait(1000);
	        session.handbrakeOff();	        
	        
	        // Hopefully deadlocked.
	        t.join(2000);
	        t1.join(2000);
	        
	        int testMsgCount = 0;
	        for(Message msg:application.toAdminMessages){
	        	
	        	if(msg.isSetField(TestReqID.FIELD) && "SessionResetTest".equals(msg.getString(TestReqID.FIELD))){
	        		testMsgCount++;
	        	}
	        }
	        Assert.assertEquals(2, testMsgCount);
	    }
	 
	 
	    private Message createAdminMessage(int sequence) {
	        final TestRequest msg = new TestRequest(new TestReqID("SessionResetTest"));
	        msg.getHeader().setString(SenderCompID.FIELD, "TARGET");
	        msg.getHeader().setString(TargetCompID.FIELD, "SENDER");
	        msg.getHeader().setInt(MsgSeqNum.FIELD, sequence);
	        msg.getHeader().setUtcTimeStamp(SendingTime.FIELD, new Date());
	        return msg;
	    }
	    
	    private class UnitTestResponder implements Responder {
	        public String sentMessageData;
	        public boolean disconnectCalled;

	        public boolean send(String data) {
	            sentMessageData = data;
	            return true;
	        }

	        public String getRemoteIPAddress() {
	            return null;
	        }

	        public void disconnect() {
	            disconnectCalled = true;
	        }
	    }

}
